unit FormPathEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Buttons;

type
  TFrmPathEdit = class(TForm)
    Panel1: TPanel;
    btnOK: TButton;
    btnCancel: TButton;
    btnHelp: TButton;
    GroupBox1: TGroupBox;
    ListBox: TListBox;
    btnAdd: TButton;
    btnDelete: TButton;
    btnReplace: TButton;
    btnDeleteInvalidPaths: TButton;
    Label1: TLabel;
    btnUp: TSpeedButton;
    btnDown: TSpeedButton;
    Label2: TLabel;
    DirectoryName: TEdit;
    Button1: TButton;
    procedure btnReplaceClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnUpClick(Sender: TObject);
    procedure btnDownClick(Sender: TObject);
    procedure btnDeleteInvalidPathsClick(Sender: TObject);
    procedure ListBoxClick(Sender: TObject);
    procedure DirectoryNameChange(Sender: TObject);
    procedure ListBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure Button1Click(Sender: TObject);
  private
		{ Private declarations }
    function GetPath: string;
    procedure SetPath(Value: string);
  public
		{ Public declarations }
    function ValidateDir(DirName: string): boolean;
    property path: string read GetPath write SetPath;
  end;

var
  FrmPathEdit: TFrmPathEdit;

implementation
uses
	Registry, FileUtil;
	
{$R *.DFM}

function TFrmPathEdit.ValidateDir(DirName: string): boolean;
begin
	result := ValidFileName( DirName ) and DirExists( DirName );
end;

procedure TFrmPathEdit.btnReplaceClick(Sender: TObject);
begin
	ListBox.Items[ListBox.ItemIndex] := DirectoryName.Text;
end;

procedure TFrmPathEdit.btnAddClick(Sender: TObject);
begin
	ListBox.Items.Add( DirectoryName.Text );
  ListBox.ItemIndex := ListBox.Items.Count -1;
  ListBoxClick( ListBox );
end;

procedure TFrmPathEdit.btnDeleteClick(Sender: TObject);
begin
	ListBox.Items.Delete(ListBox.ItemIndex);
  ListBoxClick( ListBox );
end;

procedure TFrmPathEdit.btnUpClick(Sender: TObject);
begin
	with ListBox do
		if ItemIndex > 0 then
		begin
			Items.Exchange( ItemIndex, ItemIndex -1 );
			ItemIndex := ItemIndex -1;
		end;
end;

procedure TFrmPathEdit.btnDownClick(Sender: TObject);
begin
	with ListBox do
		if ItemIndex < Items.Count -1 then
		begin
			Items.Exchange( ItemIndex, ItemIndex +1 );
			ItemIndex := ItemIndex +1;
		end;
end;

procedure TFrmPathEdit.btnDeleteInvalidPathsClick(Sender: TObject);
var
	i: integer;
begin
	i := 0;
  while i < ListBox.Items.Count do
  begin
  	if not ValidateDir( ListBox.Items[i] ) then
    	ListBox.Items.Delete( i )
    else
    	Inc( i );
  end;
end;

function TFrmPathEdit.GetPath: string;
var
	i: integer;
begin
	result := '';
	for i := 0 to ListBox.Items.Count -1 do
		result := result +';' +ListBox.Items[i];
	Delete( result, 1, 1 );
end;

procedure TFrmPathEdit.SetPath(Value: string);
begin
	if trim( Value ) <> '' then
  begin
		Value := Value +';';
  	while Value <> '' do
	  begin
  		ListBox.Items.Add( copy( Value, 1, pos( ';', Value ) -1 ) );
    	delete( Value, 1, pos( ';', Value ) );
	  end;
  end;
  if ListBox.Items.Count > 0 then
  begin
  	ListBox.ItemIndex := 0;
  end;
  ListBoxClick( ListBox );
end;

procedure TFrmPathEdit.ListBoxClick(Sender: TObject);
begin
	if ListBox.ItemIndex >= 0 then
		DirectoryName.Text := ListBox.Items[ListBox.ItemIndex]
  else
  	DirectoryName.Text := '';
	btnDelete.Enabled := (ListBox.ItemIndex >= 0);
end;

procedure TFrmPathEdit.DirectoryNameChange(Sender: TObject);
var
	i: integer;
begin
	for i := 0 to ListBox.Items.Count -1 do
  	if AnsiCompareText( DirectoryName.Text, ListBox.Items[i] ) = 0 then
    begin
    	btnReplace.Enabled := false;
      btnAdd.Enabled := false;
      exit;
    end;

  btnReplace.Enabled := (trim( DirectoryName.Text ) <> '') and (ListBox.ItemIndex >= 0);
  btnAdd.Enabled := (trim( DirectoryName.Text ) <> '');
end;

procedure TFrmPathEdit.ListBoxDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
	with (control as TListBox), (control as TListBox).Canvas do
  begin
  	FillRect( Rect );
    if (not ValidateDir( Items[Index] )) and (not (odSelected in State)) then
    	Font.Color := clGray;
    InflateRect( Rect, -2, 0 );
    DrawText( Handle, PChar( Items[Index] ), length( Items[Index] ), Rect, DT_VCENTER );
  end;
end;

procedure TFrmPathEdit.Button1Click(Sender: TObject);
var
	FolderName: string;
begin
	FolderName := DirectoryName.Text;
	if BrowseDirectory( FolderName, 'Directories', 0 ) then
  	DirectoryName.Text := FolderName;
end;

end.
